@@ -1,8 +1,11 @@
 package main
 
 import (
+	"math"
+
 	"github.com/projectdiscovery/katana/pkg/engine/standard"
 	"github.com/projectdiscovery/katana/pkg/types"
+	"github.com/projectdiscovery/katana/pkg/utils/queue"
 )
 
 var libraryTestcases = map[string]TestCase{
@@ -17,9 +20,10 @@ func (h *goIntegrationTest) Execute() error {
 	options := &types.Options{
 		MaxDepth:     1,
 		FieldScope:   "rdn",
-		BodyReadSize: 2 * 1024 * 1024,
+		BodyReadSize: math.MaxInt,
 		RateLimit:    150,
 		Verbose:      debug,
+		Strategy:     queue.DepthFirst.String(),
 	}
 	crawlerOptions, err := types.NewCrawlerOptions(options)
 	if err != nil {
