@@ -5,20 +5,21 @@ import (
 	"context"
 	"io"
 	"net/http"
+	"net/http/httputil"
 	"strings"
 
 	"github.com/PuerkitoBio/goquery"
-	"github.com/pkg/errors"
 	"github.com/projectdiscovery/katana/pkg/navigation"
 	"github.com/projectdiscovery/katana/pkg/utils"
 	"github.com/projectdiscovery/retryablehttp-go"
+	errorutil "github.com/projectdiscovery/utils/errors"
+	mapsutil "github.com/projectdiscovery/utils/maps"
 )
 
 // makeRequest makes a request to a URL returning a response interface.
-func (c *Crawler) makeRequest(ctx context.Context, request navigation.Request, rootHostname string, depth int, httpclient *retryablehttp.Client) (navigation.Response, error) {
+func (c *Crawler) makeRequest(ctx context.Context, request *navigation.Request, rootHostname string, depth int, httpclient *retryablehttp.Client) (navigation.Response, error) {
 	response := navigation.Response{
 		Depth:        request.Depth + 1,
-		Options:      c.options,
 		RootHostname: rootHostname,
 	}
 	ctx = context.WithValue(ctx, navigation.Depth{}, depth)
@@ -47,11 +48,15 @@ func (c *Crawler) makeRequest(ctx context.Context, request navigation.Request, r
 	if resp != nil {
 		defer func() {
 			if resp.Body != nil && resp.StatusCode != http.StatusSwitchingProtocols {
-				_, _ = io.CopyN(io.Discard, resp.Body, 8*1024)
+				_, _ = io.Copy(io.Discard, resp.Body)
 			}
 			_ = resp.Body.Close()
 		}()
 	}
+
+	rawRequestBytes, _ := httputil.DumpRequestOut(req.Request, true)
+	request.Raw = string(rawRequestBytes)
+
 	if err != nil {
 		return response, err
 	}
@@ -67,14 +72,25 @@ func (c *Crawler) makeRequest(ctx context.Context, request navigation.Request, r
 		return navigation.Response{}, nil
 	}
 
-	resp.Body = io.NopCloser(strings.NewReader(string(data)))
-	_ = c.options.OutputWriter.Write(nil, resp)
+	technologies := c.options.Wappalyzer.Fingerprint(resp.Header, data)
+	response.Technologies = mapsutil.GetKeys(technologies)
 
-	response.Body = data
+	resp.Body = io.NopCloser(strings.NewReader(string(data)))
+
+	response.Body = string(data)
 	response.Resp = resp
 	response.Reader, err = goquery.NewDocumentFromReader(bytes.NewReader(data))
+	response.StatusCode = resp.StatusCode
+	response.Headers = utils.FlattenHeaders(resp.Header)
+
+	resp.ContentLength = int64(len(data))
+
+	rawResponseBytes, _ := httputil.DumpResponse(resp, true)
+	response.Raw = string(rawResponseBytes)
+
 	if err != nil {
-		return response, errors.Wrap(err, "could not make document from reader")
+		return response, errorutil.NewWithTag("standard", "could not make document from reader").Wrap(err)
 	}
+
 	return response, nil
 }
