@@ -2,15 +2,17 @@ package types
 
 import (
 	"context"
+	"net/url"
 	"time"
 
-	"github.com/pkg/errors"
 	"github.com/projectdiscovery/fastdialer/fastdialer"
 	"github.com/projectdiscovery/katana/pkg/output"
 	"github.com/projectdiscovery/katana/pkg/utils/extensions"
 	"github.com/projectdiscovery/katana/pkg/utils/filters"
 	"github.com/projectdiscovery/katana/pkg/utils/scope"
 	"github.com/projectdiscovery/ratelimit"
+	errorutil "github.com/projectdiscovery/utils/errors"
+	wappalyzer "github.com/projectdiscovery/wappalyzergo"
 )
 
 // CrawlerOptions contains helper utilities for the crawler
@@ -29,6 +31,8 @@ type CrawlerOptions struct {
 	ScopeManager *scope.Manager
 	// Dialer is instance of the dialer for global crawler
 	Dialer *fastdialer.Dialer
+	// Wappalyzer instance for technologies detection
+	Wappalyzer *wappalyzer.Wappalyze
 }
 
 // NewCrawlerOptions creates a new crawler options structure
@@ -36,17 +40,21 @@ type CrawlerOptions struct {
 func NewCrawlerOptions(options *Options) (*CrawlerOptions, error) {
 	extensionsValidator := extensions.NewValidator(options.ExtensionsMatch, options.ExtensionFilter)
 
-	fastdialerInstance, err := fastdialer.NewDialer(fastdialer.DefaultOptions)
+	dialerOpts := fastdialer.DefaultOptions
+	if len(options.Resolvers) > 0 {
+		dialerOpts.BaseResolvers = options.Resolvers
+	}
+	fastdialerInstance, err := fastdialer.NewDialer(dialerOpts)
 	if err != nil {
 		return nil, err
 	}
 	scopeManager, err := scope.NewManager(options.Scope, options.OutOfScope, options.FieldScope, options.NoScope)
 	if err != nil {
-		return nil, errors.Wrap(err, "could not create scope manager")
+		return nil, errorutil.NewWithErr(err).Msgf("could not create scope manager")
 	}
 	itemFilter, err := filters.NewSimple()
 	if err != nil {
-		return nil, errors.Wrap(err, "could not create filter")
+		return nil, errorutil.NewWithErr(err).Msgf("could not create filter")
 	}
 
 	outputOptions := output.Options{
@@ -60,28 +68,35 @@ func NewCrawlerOptions(options *Options) (*CrawlerOptions, error) {
 		StoreResponseDir: options.StoreResponseDir,
 		FieldConfig:      options.FieldConfig,
 		ErrorLogFile:     options.ErrorLogFile,
+		MatchRegex:       options.MatchRegex,
+		FilterRegex:      options.FilterRegex,
 	}
 	outputWriter, err := output.New(outputOptions)
 	if err != nil {
-		return nil, errors.Wrap(err, "could not create output writer")
-	}
-
-	var ratelimiter ratelimit.Limiter
-	if options.RateLimit > 0 {
-		ratelimiter = *ratelimit.New(context.Background(), uint(options.RateLimit), time.Second)
-	} else if options.RateLimitMinute > 0 {
-		ratelimiter = *ratelimit.New(context.Background(), uint(options.RateLimitMinute), time.Minute)
+		return nil, errorutil.NewWithErr(err).Msgf("could not create output writer")
 	}
 
 	crawlerOptions := &CrawlerOptions{
 		ExtensionsValidator: extensionsValidator,
 		ScopeManager:        scopeManager,
 		UniqueFilter:        itemFilter,
-		RateLimit:           ratelimiter,
 		Options:             options,
 		Dialer:              fastdialerInstance,
 		OutputWriter:        outputWriter,
 	}
+
+	if options.RateLimit > 0 {
+		crawlerOptions.RateLimit = *ratelimit.New(context.Background(), uint(options.RateLimit), time.Second)
+	} else if options.RateLimitMinute > 0 {
+		crawlerOptions.RateLimit = *ratelimit.New(context.Background(), uint(options.RateLimitMinute), time.Minute)
+	}
+
+	wappalyze, err := wappalyzer.New()
+	if err != nil {
+		return nil, err
+	}
+	crawlerOptions.Wappalyzer = wappalyze
+
 	return crawlerOptions, nil
 }
 
@@ -90,3 +105,22 @@ func (c *CrawlerOptions) Close() error {
 	c.UniqueFilter.Close()
 	return c.OutputWriter.Close()
 }
+
+func (c *CrawlerOptions) ValidatePath(path string) bool {
+	if c.ExtensionsValidator != nil {
+		return c.ExtensionsValidator.ValidatePath(path)
+	}
+	return true
+}
+
+// ValidateScope validates scope for an AbsURL
+func (c *CrawlerOptions) ValidateScope(absURL, rootHostname string) (bool, error) {
+	parsed, err := url.Parse(absURL)
+	if err != nil {
+		return false, err
+	}
+	if c.ScopeManager != nil {
+		return c.ScopeManager.Validate(parsed, rootHostname)
+	}
+	return true, nil
+}
